'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const sdk_1 = require("./sdk");
const configuration_1 = require("../../configuration");
const typescript_ioc_1 = require("typescript-ioc");
const fs = require("fs-extra-promise");
const YAML = require("yamljs");
const config_1 = require("../../utils/config");
class Cli {
    constructor(configArgs) {
        this.args = configArgs;
    }
    async processCommand() {
        this.sdk = await sdk_1.SDK.initialize({
            defaultHost: config_1.getSwaggerHost(this.config.gateway),
            swaggerUrl: config_1.getSwaggerUrl(this.config.gateway),
            token: config_1.generateSecurityToken(this.config.gateway)
        });
        return this.doCommand();
    }
    async doCommand() {
        switch (this.args.command) {
            case 'apis':
                return this.processApis();
            case 'gateway':
                return this.processGateway();
            case 'config':
                return this.processConfig();
            case 'users':
                return this.processUsers();
            case 'middleware':
                return this.processMiddleware();
            default:
                throw new Error(`Command not found: ${this.args.command}`);
        }
    }
    async processUsers() {
        switch (this.args.usersCommand) {
            case 'add':
                return this.processUsersAdd();
            case 'remove':
                return this.processUsersRemove();
            case 'list':
                return this.processUsersList();
            case 'password':
                return this.processUsersPassword();
            case 'get':
                return this.processUsersGet();
            case 'update':
                return this.processUsersUpdate();
            default:
                throw new Error(`Command not found: ${this.args.usersCommand}`);
        }
    }
    async processUsersAdd() {
        const user = {
            login: this.args.login,
            name: this.args.name,
            password: this.args.password,
            roles: []
        };
        if (this.args.email) {
            user.email = this.args.email;
        }
        if (this.args.roles) {
            this.args.roles.forEach((role) => {
                if (role === 'config' || role === 'admin') {
                    user.roles.push(role);
                }
                else {
                    console.info(`Invalid role ${role}. Ignoring it...`);
                }
            });
        }
        await this.sdk.users.addUser(user);
        console.info('User created.');
    }
    async processUsersUpdate() {
        const user = {
            login: this.args.login,
        };
        if (this.args.name) {
            user.name = this.args.name;
        }
        if (this.args.email) {
            user.email = this.args.email;
        }
        if (this.args.roles) {
            user.roles = [];
            this.args.roles.forEach((role) => {
                if (role === 'config' || role === 'admin') {
                    user.roles.push(role);
                }
                else {
                    console.info(`Invalid role ${role}. Ignoring it...`);
                }
            });
        }
        await this.sdk.users.updateUser(user.login, user);
        console.info(`User updated`);
    }
    async processUsersRemove() {
        await this.sdk.users.removeUser(this.args.login);
        console.info('User removed.');
    }
    async processUsersGet() {
        const user = await this.sdk.users.getUser(this.args.login);
        if (this.args.format === 'json') {
            console.info(JSON.stringify(user, null, 4));
        }
        else {
            console.info(YAML.stringify(user, 15));
        }
    }
    async processUsersList() {
        this.args.filter = this.args.filter || [];
        const args = {};
        this.args.filter.forEach((param) => {
            const parts = param.split(':');
            if (parts.length === 2) {
                args[parts[0]] = parts[1];
            }
        });
        const users = await this.sdk.users.list(args);
        console.info(YAML.stringify(users));
    }
    async processUsersPassword() {
        await this.sdk.users.changeUserPassword(this.args.login, this.args.password);
        console.info('Password changed.');
    }
    async processMiddleware() {
        switch (this.args.middlewareCommand) {
            case 'filter':
                return this.processMiddlewareFilter();
            case 'requestInterceptor':
                return this.processMiddlewareRequestInterceptor();
            case 'responseInterceptor':
                return this.processMiddlewareResponseInterceptor();
            case 'authStrategy':
                return this.processMiddlewareAuthStrategy();
            case 'authVerify':
                return this.processMiddlewareAuthVerify();
            case 'throttlingKeyGenerator':
                return this.processMiddlewareThrottlingKeyGenerator();
            case 'throttlingHandler':
                return this.processMiddlewareThrottlingHandler();
            case 'throttlingSkip':
                return this.processMiddlewareThrottlingSkip();
            case 'circuitbreaker':
                return this.processMiddlewareCircuitBreaker();
            case 'cors':
                return this.processMiddlewareCors();
            case 'proxyRouter':
                return this.processMiddlewareProxyRouter();
            case 'serviceDiscovery':
                return this.processMiddlewareServiceDiscovery();
            case 'serviceDiscoveryProvider':
                return this.processMiddlewareServiceDiscoveryProvider();
            case 'errorHandler':
                return this.processMiddlewareErrorHandler();
            case 'requestLogger':
                return this.processMiddlewareRequestLogger();
            default:
                throw new Error(`Command not found: ${this.args.command}`);
        }
    }
    async processMiddlewareFilter() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.filters(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeFilter(this.args.remove);
            console.info(`Filter removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateFilter(name, fileName);
            console.info(`Filter updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addFilter(name, fileName);
            console.info(`Filter added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getFilter(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareRequestInterceptor() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.requestInterceptors(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeRequestInterceptor(this.args.remove);
            console.info(`Interceptor removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateRequestInterceptor(name, fileName);
            console.info(`Interceptor updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addRequestInterceptor(name, fileName);
            console.info(`Interceptor added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getRequestInterceptor(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareResponseInterceptor() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.responseInterceptors(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeResponseInterceptor(this.args.remove);
            console.info(`Interceptor removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateResponseInterceptor(name, fileName);
            console.info(`Interceptor updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addResponseInterceptor(name, fileName);
            console.info(`Interceptor added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getResponseInterceptor(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareAuthStrategy() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.authStrategies(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeAuthStrategy(this.args.remove);
            console.info(`Interceptor removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateAuthStrategy(name, fileName);
            console.info(`Interceptor updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addAuthStrategy(name, fileName);
            console.info(`Interceptor added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getAuthStrategy(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareAuthVerify() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.authVerify(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeAuthVerify(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateAuthVerify(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addAuthVerify(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getAuthVerify(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareThrottlingKeyGenerator() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.throttlingKeyGenerator(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeThrottlingKeyGenerator(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateThrottlingKeyGenerator(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addThrottlingKeyGenerator(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getThrottlingKeyGenerator(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareThrottlingHandler() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.throttlingHandler(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeThrottlingHandler(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateThrottlingHandler(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addThrottlingHandler(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getThrottlingHandler(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareThrottlingSkip() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.throttlingSkip(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeThrottlingSkip(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateThrottlingSkip(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addThrottlingSkip(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getThrottlingSkip(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareCircuitBreaker() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.circuitBreaker(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeCircuitBreaker(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateCircuitBreaker(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addCircuitBreaker(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getCircuitBreaker(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareCors() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.corsOrigin(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeCors(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateCors(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addCors(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getCors(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareProxyRouter() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.proxyRouter(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeProxyRouter(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateProxyRouter(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addProxyRouter(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getProxyRouter(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareServiceDiscovery() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.serviceDiscovery(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeServiceDiscovery(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateServiceDiscovery(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addServiceDiscovery(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getServiceDiscovery(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareServiceDiscoveryProvider() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.serviceDiscoveryProvider(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeServiceDiscoveryProvider(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateServiceDiscoveryProvider(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addServiceDiscoveryProvider(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getServiceDiscoveryProvider(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareErrorHandler() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.errorHandler(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeErrorHandler(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateErrorHandler(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addErrorHandler(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getErrorHandler(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processMiddlewareRequestLogger() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const filters = await this.sdk.middleware.requestLogger(args['name']);
            console.info(YAML.stringify(filters));
        }
        else if (this.args.remove) {
            await this.sdk.middleware.removeRequestLogger(this.args.remove);
            console.info(`Middleware removed`);
        }
        else if (this.args.update) {
            const name = this.args.update[0];
            const fileName = this.args.update[1];
            await this.sdk.middleware.updateRequestLogger(name, fileName);
            console.info(`Middleware updated`);
        }
        else if (this.args.add) {
            const name = this.args.add[0];
            const fileName = this.args.add[1];
            await this.sdk.middleware.addRequestLogger(name, fileName);
            console.info(`Middleware added`);
        }
        else if (this.args.get) {
            const file = await this.sdk.middleware.getRequestLogger(this.args.get);
            console.info(file.toString());
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processApis() {
        if (this.args.list) {
            const args = {};
            this.args.list.forEach((param) => {
                const parts = param.split(':');
                if (parts.length === 2) {
                    args[parts[0]] = parts[1];
                }
            });
            const apis = await this.sdk.apis.list(args);
            console.info(YAML.stringify(apis));
        }
        else if (this.args.add) {
            const api = await this.loadConfigObject(this.args.add);
            const apiId = await this.sdk.apis.addApi(api);
            console.info(`API created. ID: ${apiId}`);
            if (api.id !== apiId) {
                api.id = apiId;
                await this.updateApiConfig(this.args.add, api);
            }
        }
        else if (this.args.update) {
            const api = await this.loadConfigObject(this.args.update);
            await this.sdk.apis.updateApi(api.id, api);
            console.info(`API updated`);
        }
        else if (this.args.remove) {
            await this.sdk.apis.removeApi(this.args.remove);
            console.info(`API removed`);
        }
        else if (this.args.get) {
            const id = this.args.get[0];
            const format = this.args.get.length > 1 ? this.args.get[1] : 'yaml';
            const api = await this.sdk.apis.getApi(id);
            if (format === 'json') {
                console.info(JSON.stringify(api, null, 4));
            }
            else {
                console.info(YAML.stringify(api, 15));
            }
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processGateway() {
        if (this.args.update) {
            const gateway = await this.loadConfigObject(this.args.update);
            await this.sdk.gateway.updateConfig(gateway);
            console.info(`Gateway config updated`);
        }
        else if (this.args.remove) {
            await this.sdk.gateway.removeConfig();
            console.info(`Gateway config removed`);
        }
        else if (this.args.get) {
            const gateway = await this.sdk.gateway.getConfig();
            if (this.args.get === 'json') {
                console.info(JSON.stringify(gateway, null, 4));
            }
            else {
                console.info(YAML.stringify(gateway, 15));
            }
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async processConfig() {
        if (this.args.update) {
            const config = await this.loadConfigObject(this.args.update);
            await this.sdk.config.set(config);
            console.info(`Gateway configurations imported`);
        }
        else if (this.args.get) {
            const config = await this.sdk.config.get();
            if (this.args.get === 'json') {
                console.info(JSON.stringify(config, null, 4));
            }
            else {
                console.info(YAML.stringify(config, 15));
            }
        }
        else {
            throw new Error('Invalid arguments. Type -h for more info.');
        }
    }
    async loadConfigObject(fileName) {
        const nameLowerCase = fileName.toLowerCase();
        if (nameLowerCase.endsWith('.yml') || nameLowerCase.endsWith('.yaml')) {
            return YAML.load(fileName);
        }
        else {
            return await fs.readJSONAsync(fileName);
        }
    }
    async updateApiConfig(fileName, api) {
        const nameLowerCase = fileName.toLowerCase();
        if (nameLowerCase.endsWith('.yml') || nameLowerCase.endsWith('.yaml')) {
            return await fs.writeFileAsync(fileName, YAML.stringify(api, 15));
        }
        else {
            return await fs.writeJSONAsync(fileName, api);
        }
    }
}
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", configuration_1.Configuration)
], Cli.prototype, "config", void 0);
exports.Cli = Cli;
//# sourceMappingURL=cli-tool.js.map