'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const fs = require("fs-extra-promise");
const path = require("path");
const request = require("request");
const utils_1 = require("./utils");
class MiddlewareClient {
    constructor(swaggerClient, authToken) {
        this.swaggerClient = swaggerClient;
        this.authToken = authToken;
        if (!this.swaggerClient.spec || !this.swaggerClient.spec.schemes ||
            !this.swaggerClient.spec.schemes.length || !this.swaggerClient.spec.host) {
            throw new Error('Invalid swagger specification. Can not found the target endpoint to call.');
        }
        this.middlewareRequest = request.defaults({ baseUrl: `${this.swaggerClient.spec.schemes[0]}://${this.swaggerClient.spec.host}` });
    }
    async filters(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestFilters({ name }));
        return utils_1.getResponseBody(response);
    }
    async requestInterceptors(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRequestInterceptors({ name }));
        return utils_1.getResponseBody(response);
    }
    async responseInterceptors(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestResponseInterceptors({ name }));
        return utils_1.getResponseBody(response);
    }
    async authStrategies(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestAuthStrategies({ name }));
        return utils_1.getResponseBody(response);
    }
    async authVerify(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestAuthVerify({ name }));
        return utils_1.getResponseBody(response);
    }
    async throttlingKeyGenerator(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestThrottlingKeyGenerator({ name }));
        return utils_1.getResponseBody(response);
    }
    async throttlingHandler(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestThrottlingHandler({ name }));
        return utils_1.getResponseBody(response);
    }
    async throttlingSkip(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestThrottlingSkip({ name }));
        return utils_1.getResponseBody(response);
    }
    async circuitBreaker(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestCircuitBreaker({ name }));
        return utils_1.getResponseBody(response);
    }
    async corsOrigin(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestCorsOrigin({ name }));
        return utils_1.getResponseBody(response);
    }
    async proxyRouter(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestProxyRouter({ name }));
        return utils_1.getResponseBody(response);
    }
    async serviceDiscovery(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestServiceDiscovery({ name }));
        return utils_1.getResponseBody(response);
    }
    async serviceDiscoveryProvider(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestServiceDiscoveryProvider({ name }));
        return utils_1.getResponseBody(response);
    }
    async errorHandler(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestErrorHandler({ name }));
        return utils_1.getResponseBody(response);
    }
    async requestLogger(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRequestLogger({ name }));
        return utils_1.getResponseBody(response);
    }
    async removeFilter(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveFilter({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeRequestInterceptor(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveRequestInterceptor({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeResponseInterceptor(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveResponseInterceptor({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeAuthStrategy(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveAuthStrategy({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeAuthVerify(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveAuthVerify({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeThrottlingKeyGenerator(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveThrottlingKeyGenerator({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeThrottlingHandler(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveThrottlingHandler({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeThrottlingSkip(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveThrottlingSkip({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeCircuitBreaker(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveCircuitBreaker({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeCors(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveCors({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeProxyRouter(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveProxyRouter({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeServiceDiscovery(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveServiceDiscovery({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeServiceDiscoveryProvider(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveServiceDiscoveryProvider({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeErrorHandler(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveErrorHandler({ name }));
        utils_1.checkStatus(response, 204);
    }
    async removeRequestLogger(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestRemoveRequestLogger({ name }));
        utils_1.checkStatus(response, 204);
    }
    updateFilter(name, fileName) {
        return this.updateMiddleware('filters', fileName, name);
    }
    updateRequestInterceptor(name, fileName) {
        return this.updateMiddleware('interceptors/request', fileName, name);
    }
    updateResponseInterceptor(name, fileName) {
        return this.updateMiddleware('interceptors/response', fileName, name);
    }
    updateAuthStrategy(name, fileName) {
        return this.updateMiddleware('authentication/strategies', fileName, name);
    }
    updateAuthVerify(name, fileName) {
        return this.updateMiddleware('authentication/verify', fileName, name);
    }
    updateThrottlingKeyGenerator(name, fileName) {
        return this.updateMiddleware('throttling/keyGenerators', fileName, name);
    }
    updateThrottlingHandler(name, fileName) {
        return this.updateMiddleware('throttling/handlers', fileName, name);
    }
    updateThrottlingSkip(name, fileName) {
        return this.updateMiddleware('throttling/skip', fileName, name);
    }
    updateCircuitBreaker(name, fileName) {
        return this.updateMiddleware('circuitbreaker', fileName, name);
    }
    updateCors(name, fileName) {
        return this.updateMiddleware('cors/origin', fileName, name);
    }
    updateProxyRouter(name, fileName) {
        return this.updateMiddleware('proxy/router', fileName, name);
    }
    updateServiceDiscovery(name, fileName) {
        return this.updateMiddleware('servicediscovery', fileName, name);
    }
    updateServiceDiscoveryProvider(name, fileName) {
        return this.updateMiddleware('servicediscovery/provider', fileName, name);
    }
    updateErrorHandler(name, fileName) {
        return this.updateMiddleware('errorhandler', fileName, name);
    }
    updateRequestLogger(name, fileName) {
        return this.updateMiddleware('request/logger', fileName, name);
    }
    addFilter(name, fileName) {
        return this.installMiddleware('filters', fileName, name);
    }
    addRequestInterceptor(name, fileName) {
        return this.installMiddleware('interceptors/request', fileName, name);
    }
    addResponseInterceptor(name, fileName) {
        return this.installMiddleware('interceptors/response', fileName, name);
    }
    addAuthStrategy(name, fileName) {
        return this.installMiddleware('authentication/strategies', fileName, name);
    }
    addAuthVerify(name, fileName) {
        return this.installMiddleware('authentication/verify', fileName, name);
    }
    addThrottlingKeyGenerator(name, fileName) {
        return this.installMiddleware('throttling/keyGenerators', fileName, name);
    }
    addThrottlingHandler(name, fileName) {
        return this.installMiddleware('throttling/handlers', fileName, name);
    }
    addThrottlingSkip(name, fileName) {
        return this.installMiddleware('throttling/skip', fileName, name);
    }
    addCircuitBreaker(name, fileName) {
        return this.installMiddleware('circuitbreaker', fileName, name);
    }
    addCors(name, fileName) {
        return this.installMiddleware('cors/origin', fileName, name);
    }
    addProxyRouter(name, fileName) {
        return this.installMiddleware('proxy/router', fileName, name);
    }
    addServiceDiscovery(name, fileName) {
        return this.installMiddleware('servicediscovery', fileName, name);
    }
    addServiceDiscoveryProvider(name, fileName) {
        return this.installMiddleware('servicediscovery/provider', fileName, name);
    }
    addErrorHandler(name, fileName) {
        return this.installMiddleware('errorhandler', fileName, name);
    }
    addRequestLogger(name, fileName) {
        return this.installMiddleware('request/logger', fileName, name);
    }
    async getFilter(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetFilter({ name }));
        return utils_1.getResponseBody(response);
    }
    async getRequestInterceptor(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetRequestInterceptor({ name }));
        return utils_1.getResponseBody(response);
    }
    async getResponseInterceptor(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetResponseInterceptor({ name }));
        return utils_1.getResponseBody(response);
    }
    async getAuthStrategy(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetAuthStrategy({ name }));
        return utils_1.getResponseBody(response);
    }
    async getAuthVerify(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetAuthVerify({ name }));
        return utils_1.getResponseBody(response);
    }
    async getThrottlingKeyGenerator(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetThrottlingKeyGenerator({ name }));
        return utils_1.getResponseBody(response);
    }
    async getThrottlingHandler(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetThrottlingHandler({ name }));
        return utils_1.getResponseBody(response);
    }
    async getThrottlingSkip(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetThrottlingSkip({ name }));
        return utils_1.getResponseBody(response);
    }
    async getCircuitBreaker(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetCircuitBreakerMiddleware({ name }));
        return utils_1.getResponseBody(response);
    }
    async getCors(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetCorsMiddleware({ name }));
        return utils_1.getResponseBody(response);
    }
    async getProxyRouter(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetProxyRouterMiddleware({ name }));
        return utils_1.getResponseBody(response);
    }
    async getServiceDiscovery(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetServiceDiscoveryMiddleware({ name }));
        return utils_1.getResponseBody(response);
    }
    async getServiceDiscoveryProvider(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetServiceDiscoveryProviderMiddleware({ name }));
        return utils_1.getResponseBody(response);
    }
    async getErrorHandler(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetErrorHandlerMiddleware({ name }));
        return utils_1.getResponseBody(response);
    }
    async getRequestLogger(name) {
        const response = await utils_1.invoke(this.swaggerClient.apis.Middleware.MiddlewareRestGetRequestLoggerMiddleware({ name }));
        return utils_1.getResponseBody(response);
    }
    getStream(fileName) {
        if (_.startsWith(fileName, '.')) {
            fileName = path.join(process.cwd(), fileName);
        }
        return fs.createReadStream(fileName);
    }
    installMiddleware(servicePath, filePath, name) {
        return new Promise((resolve, reject) => {
            const stream = this.getStream(filePath);
            const fileName = filePath.substring(filePath.lastIndexOf('/') + 1);
            const req = this.middlewareRequest.post('/middleware/' + servicePath, {
                headers: { 'authorization': `Bearer ${this.authToken}` }
            }, (error, response, body) => {
                if (error) {
                    return reject(error);
                }
                if (response.statusCode === 201) {
                    return resolve();
                }
                return reject(`Status code: ${response.statusCode}`);
            });
            const form = req.form();
            form.append('name', name);
            form.append('file', stream, fileName);
        });
    }
    updateMiddleware(servicePath, filePath, name) {
        return new Promise((resolve, reject) => {
            const stream = this.getStream(filePath);
            const fileName = filePath.substring(filePath.lastIndexOf('/') + 1);
            const req = this.middlewareRequest.put(`/middleware/${servicePath}/${name}`, {
                headers: { 'authorization': `Bearer ${this.authToken}` }
            }, (error, response, body) => {
                if (error) {
                    return reject(error);
                }
                if (response.statusCode === 204) {
                    return resolve();
                }
                return reject(`Status code: ${response.statusCode}`);
            });
            const form = req.form();
            form.append('file', stream, fileName);
        });
    }
}
exports.MiddlewareClient = MiddlewareClient;
//# sourceMappingURL=sdk-middleware.js.map