'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
class UsersClient {
    constructor(swaggerClient) {
        this.swaggerClient = swaggerClient;
    }
    async list(filters) {
        const response = await this.swaggerClient.apis.Users.UsersRestList(filters);
        return utils_1.getResponseBody(response);
    }
    async addUser(user) {
        const response = await this.swaggerClient.apis.Users.UsersRestAddUser({ user: user });
        return utils_1.getCreatedResource(response).substring(6);
    }
    async updateUser(login, user) {
        user.login = login;
        const response = await this.swaggerClient.apis.Users.UsersRestUpdateUser({ login: login, user: user });
        utils_1.checkStatus(response, 204);
    }
    async removeUser(login) {
        const response = await this.swaggerClient.apis.Users.UsersRestRemoveUser({ login: login });
        utils_1.checkStatus(response, 204);
    }
    async getUser(login) {
        const response = await this.swaggerClient.apis.Users.UsersRestGetUser({ login: login });
        return utils_1.getResponseBody(response);
    }
    async changeUserPassword(login, password) {
        const response = await this.swaggerClient.apis.Users.UsersRestChangePassword({ login: login, password: password });
        utils_1.checkStatus(response, 204);
    }
}
exports.UsersClient = UsersClient;
//# sourceMappingURL=sdk-users.js.map