'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const sdk_apis_1 = require("./sdk-apis");
const sdk_config_package_1 = require("./sdk-config-package");
const sdk_users_1 = require("./sdk-users");
const sdk_gateway_1 = require("./sdk-gateway");
const sdk_middleware_1 = require("./sdk-middleware");
const swagger = require('swagger-client');
class SDK {
    constructor(swaggerClient, authToken) {
        this.apisClient = new sdk_apis_1.ApisClient(swaggerClient);
        this.configClient = new sdk_config_package_1.ConfigClient(swaggerClient);
        this.gatewayClient = new sdk_gateway_1.GatewayClient(swaggerClient);
        this.middlewareClient = new sdk_middleware_1.MiddlewareClient(swaggerClient, authToken);
        this.usersClient = new sdk_users_1.UsersClient(swaggerClient);
    }
    static async initialize(options) {
        if (!options || !options.token ||
            !options.swaggerUrl) {
            throw new Error('Invalid parameters. You must informa a valid authentication token and a valid URL to the gateway swagger file.');
        }
        const swaggerClient = await swagger(options.swaggerUrl, {
            authorizations: {
                Bearer: `Bearer ${options.token}`
            }
        });
        if (!swaggerClient.spec) {
            swaggerClient.spec = {};
        }
        if (!swaggerClient.spec.host && options.defaultHost) {
            swaggerClient.spec.host = options.defaultHost;
        }
        return new SDK(swaggerClient, options.token);
    }
    get apis() {
        return this.apisClient;
    }
    get config() {
        return this.configClient;
    }
    get gateway() {
        return this.gatewayClient;
    }
    get middleware() {
        return this.middlewareClient;
    }
    get users() {
        return this.usersClient;
    }
}
exports.SDK = SDK;
//# sourceMappingURL=sdk.js.map