'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("./errors");
function checkStatus(response, status) {
    if (response.status !== status) {
        throw new Error(response.text);
    }
}
exports.checkStatus = checkStatus;
function getResponseBody(response) {
    checkStatus(response, 200);
    return response.body;
}
exports.getResponseBody = getResponseBody;
function getCreatedResource(response) {
    checkStatus(response, 201);
    return response.headers['location'];
}
exports.getCreatedResource = getCreatedResource;
async function invoke(promise) {
    try {
        return await promise;
    }
    catch (err) {
        if (err.response && err.response.body) {
            throw new errors_1.SdkError(err.response.body.error, err.response.body.code);
        }
        throw new errors_1.SdkError(err.message, 500);
    }
}
exports.invoke = invoke;
//# sourceMappingURL=utils.js.map