'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const gateway_1 = require("./gateway");
const api_1 = require("./api");
const errors_1 = require("./errors");
exports.configPackageValidatorSchema = Joi.object().keys({
    apis: Joi.array().items(api_1.apiConfigValidatorSchema),
    gateway: gateway_1.gatewayConfigValidatorSchema,
    middlewares: Joi.array().items(Joi.object().keys({
        content: Joi.string().required(),
        id: Joi.string(),
        middleware: Joi.string().required(),
        name: Joi.string()
    }).xor('id', 'name'))
});
function validateConfigPackage(configPackage) {
    return new Promise((resolve, reject) => {
        Joi.validate(configPackage, exports.configPackageValidatorSchema, (err, value) => {
            if (err) {
                reject(new errors_1.ValidationError(err));
            }
            else {
                resolve(value);
            }
        });
    });
}
exports.validateConfigPackage = validateConfigPackage;
//# sourceMappingURL=config-package.js.map