'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const _ = require("lodash");
class ValidationError extends typescript_rest_1.Errors.ForbidenError {
    constructor(entity) {
        let message;
        if (entity instanceof String || typeof (entity) === 'string') {
            message = entity;
        }
        else {
            message = ValidationError.buildValidationErrorString(entity);
        }
        super(message);
        this.entity = { error: message };
        Object.setPrototypeOf(this, ValidationError.prototype);
    }
    static buildValidationErrorString(err) {
        const requiredFields = [];
        const unexpectedFields = [];
        const invalidFields = [];
        err.details.forEach(e => {
            if (_.endsWith(e.type, 'required')) {
                requiredFields.push(e.path);
            }
            else if (_.endsWith(e.type, 'Unknown')) {
                unexpectedFields.push(e.path);
            }
            else {
                invalidFields.push(e.path);
            }
        });
        const result = [];
        if (requiredFields.length > 0) {
            result.push(`Missing Required Fields: ${requiredFields.join(', ')}`);
        }
        if (unexpectedFields.length > 0) {
            result.push(`Unexpected Fields: ${unexpectedFields.join(', ')}`);
        }
        if (invalidFields.length > 0) {
            result.push(`Invalid value for Fields: ${invalidFields.join(', ')}`);
        }
        return result.join('\n');
    }
}
exports.ValidationError = ValidationError;
//# sourceMappingURL=errors.js.map