'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["error"] = 0] = "error";
    LogLevel[LogLevel["warn"] = 1] = "warn";
    LogLevel[LogLevel["info"] = 2] = "info";
    LogLevel[LogLevel["debug"] = 3] = "debug";
})(LogLevel = exports.LogLevel || (exports.LogLevel = {}));
const validLevels = Joi.string().valid('error', 'warn', 'info', 'debug');
const logConsoleConfigSchema = Joi.object().keys({
    colorize: Joi.boolean(),
    depth: Joi.number().positive(),
    humanReadableUnhandledException: Joi.boolean(),
    json: Joi.boolean(),
    level: Joi.string().valid('error', 'warn', 'info', 'debug'),
    prettyPrint: Joi.boolean(),
    showLevel: Joi.boolean(),
    silent: Joi.boolean(),
    stderrLevels: Joi.alternatives([Joi.array().items(validLevels), validLevels]),
    stringify: Joi.boolean(),
    timestamp: Joi.boolean()
});
const logFileConfigSchema = Joi.object().keys({
    colorize: Joi.boolean(),
    depth: Joi.number().positive(),
    eol: Joi.string(),
    json: Joi.boolean(),
    level: Joi.string().valid('error', 'warn', 'info', 'debug'),
    logstash: Joi.boolean(),
    maxFiles: Joi.number().positive(),
    maxRetries: Joi.number().positive(),
    maxsize: Joi.number().positive(),
    outputDir: Joi.string().regex(/^[a-z\.\/][a-zA-Z0-9\.\/]*$/),
    prettyPrint: Joi.boolean(),
    showLevel: Joi.boolean(),
    silent: Joi.boolean(),
    tailable: Joi.boolean(),
    timestamp: Joi.boolean(),
    zippedArchive: Joi.boolean()
}).with('depth', 'prettyPrint').with('tailable', 'maxFiles');
exports.loggerConfigSchema = Joi.object().keys({
    console: logConsoleConfigSchema,
    file: logFileConfigSchema,
    level: Joi.string().valid('error', 'warn', 'info', 'debug')
});
exports.accessLoggerConfigSchema = Joi.object().keys({
    console: logConsoleConfigSchema,
    expressFormat: Joi.boolean(),
    file: logFileConfigSchema,
    msg: Joi.string()
});
//# sourceMappingURL=logger.js.map