'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Winston = require("winston");
const path = require("path");
const fs = require("fs-extra-promise");
const _ = require("lodash");
const expressWinston = require('express-winston');
class AccessLogger {
    static configureAccessLoger(config, rootPath, server, defaultDir) {
        config = _.defaults(config, {
            meta: false,
            statusLevels: true
        });
        const options = _.omit(config, 'console', 'file');
        options.transports = [];
        if (config && config.console) {
            options.transports.push(new Winston.transports.Console(config.console));
        }
        if (config && config.file) {
            config.file = _.omit(config.file, 'outputDir');
            let outputDir = config.file.outputDir || defaultDir;
            if (_.startsWith(outputDir, '.')) {
                outputDir = path.join(rootPath, outputDir);
            }
            const fileName = (process.env.processNumber ? `access-${process.env.processNumber}.log` : `access.log`);
            config.file['filename'] = path.join(outputDir, fileName);
            fs.ensureDirSync(path.dirname(config.file['filename']));
            options.transports.push(new Winston.transports.File(config.file));
        }
        server.use(expressWinston.logger(options));
    }
}
exports.AccessLogger = AccessLogger;
//# sourceMappingURL=express-logger.js.map