'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const server_cache_1 = require("./server-cache");
const client_cache_1 = require("./client-cache");
const Groups = require("../group");
const _ = require("lodash");
const logger_1 = require("../../logger");
const typescript_ioc_1 = require("typescript-ioc");
const functions_1 = require("../../utils/functions");
const request_1 = require("../stats/request");
const onHeaders = require('on-headers');
let ApiCache = class ApiCache {
    cache(apiRouter, api, pipelineConfig) {
        if (this.useCache(api)) {
            this.configureCache(apiRouter, api, pipelineConfig);
        }
    }
    configureCache(apiRouter, api, pipelineConfig) {
        const path = api.path;
        const cacheConfigs = this.sortCaches(api.cache, path);
        cacheConfigs.forEach((cache) => {
            cache = this.resolveReferences(cache, pipelineConfig);
            let validateGroupFunction;
            if (cache.group) {
                if (this.logger.isDebugEnabled()) {
                    const groups = Groups.filter(api.group, cache.group);
                    this.logger.debug(`Configuring Group filters for Cache on path [${api.path}]. Groups [${JSON.stringify(groups)}]`);
                }
                validateGroupFunction = Groups.buildGroupAllowFilter(api.group, cache.group);
            }
            try {
                const cacheMiddleware = this.buildCacheMiddleware(validateGroupFunction, cache, path, api);
                apiRouter.use(cacheMiddleware);
            }
            catch (e) {
                this.logger.error(e);
            }
        });
    }
    resolveReferences(cache, pipelineConfig) {
        if (cache.use && pipelineConfig.cache) {
            if (pipelineConfig.cache[cache.use]) {
                cache = _.defaults(cache, pipelineConfig.cache[cache.use]);
            }
            else {
                throw new Error(`Invalid reference ${cache.use}. There is no configuration for this id.`);
            }
        }
        return cache;
    }
    buildCacheMiddleware(validateGroupFunction, cache, path, api) {
        const body = new Array();
        const requestLogEnabled = this.requestLogger.isRequestLogEnabled(api);
        if (validateGroupFunction) {
            body.push(`if (validateGroupFunction(req, res)){`);
        }
        else {
            body.push(`if (req.method === 'GET'){`);
        }
        if (cache.client) {
            const clientCache = new client_cache_1.ClientCache();
            body.push(clientCache.buildCacheMiddleware(cache.client, path));
        }
        if (cache.server) {
            const serverCache = new server_cache_1.ServerCache();
            if (requestLogEnabled) {
                body.push(serverCache.buildCacheMiddleware(cache.server, path, 'req', 'res', 'next', true));
            }
            else {
                body.push(serverCache.buildCacheMiddleware(cache.server, path, 'req', 'res', 'next'));
            }
        }
        body.push(`}`);
        body.push(`next();`);
        return functions_1.createFunction({
            ServerCache: server_cache_1.ServerCache,
            onHeaders: onHeaders,
            validateGroupFunction: validateGroupFunction
        }, 'req', 'res', 'next', body.join(''));
    }
    useCache(api) {
        if (api.cache && api.cache.length > 0) {
            return true;
        }
        return false;
    }
    sortCaches(caches, path) {
        const generalCaches = _.filter(caches, (value) => {
            if (value.group) {
                return true;
            }
            return false;
        });
        if (generalCaches.length > 1) {
            this.logger.error(`Invalid cache configuration for api [${path}]. Conflicting configurations for default group`);
            return [];
        }
        if (generalCaches.length > 0) {
            const index = caches.indexOf(generalCaches[0]);
            if (index < caches.length - 1) {
                const gen = caches.splice(index, 1);
                caches.push(gen[0]);
            }
        }
        return caches;
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ApiCache.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", request_1.RequestLogger)
], ApiCache.prototype, "requestLogger", void 0);
ApiCache = __decorate([
    typescript_ioc_1.AutoWired
], ApiCache);
exports.ApiCache = ApiCache;
//# sourceMappingURL=cache.js.map