'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
class RedisStore {
    constructor(options) {
        this.options = _.defaults(options, {
            maxAge: 60,
            prefix: 'ca:'
        });
    }
    set(key, value, maxAge) {
        const rdskey = this.options.prefix + key;
        const opt = this.options;
        if (maxAge) {
            opt.maxAge = maxAge / 1000;
        }
        opt.client.set(rdskey, JSON.stringify(value), 'EX', opt.maxAge);
    }
    get(key, callback) {
        const rdskey = this.options.prefix + key;
        this.options.client.get(rdskey).then((cachedValue) => {
            if (cachedValue) {
                callback(null, JSON.parse(cachedValue));
            }
            else {
                callback(null, null);
            }
        }).catch((err) => {
            callback(err, null);
        });
    }
    del(key) {
        const rdskey = this.options.prefix + key;
        this.options.client.del(rdskey);
    }
}
exports.RedisStore = RedisStore;
//# sourceMappingURL=redis-store.js.map