/// <reference types="node" />
/// <reference types="express" />
import { EventEmitter } from 'events';
import * as express from 'express';
export declare enum State {
    OPEN = 0,
    CLOSED = 1,
    HALF_OPEN = 2,
}
export interface Options {
    timeout: number;
    maxFailures: number;
    stateHandler: StateHandler;
    timeoutMessage: string;
    timeoutStatusCode: number;
    rejectMessage: string;
    rejectStatusCode: number;
    id: string;
}
export interface StateHandler {
    halfOpenCallPending: boolean;
    isOpen(): boolean;
    isHalfOpen(): boolean;
    isClosed(): boolean;
    forceOpen(): boolean;
    forceHalfOpen(): boolean;
    forceClose(): boolean;
    incrementFailures(): Promise<number>;
    initialState(): void;
    onStateChanged(state: string): void;
    destroy(): void;
}
export declare class CircuitBreaker extends EventEmitter {
    private options;
    constructor(options: Options);
    isOpen(): boolean;
    isHalfOpen(): boolean;
    isClosed(): boolean;
    forceOpen(req: express.Request): void;
    forceClosed(req: express.Request): void;
    forceHalfOpen(): void;
    middleware(): express.RequestHandler;
    onStateChanged(state: string): void;
    readonly id: string;
    destroy(): void;
    private invokeApi(req, res, next);
    private fastFail(next);
    private handleTimeout(req, res, next);
    private handleSuccess(req);
    private handleFailure(req, err);
}
