'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const Groups = require("../group");
const corsLib = require("cors");
const logger_1 = require("../../logger");
const typescript_ioc_1 = require("typescript-ioc");
const time_intervals_1 = require("../../utils/time-intervals");
const middleware_loader_1 = require("../../utils/middleware-loader");
let ApiCors = class ApiCors {
    cors(apiRouter, api, pipelineConfig) {
        const path = api.path;
        const configusrations = this.sortMiddlewares(api.cors, path);
        const corsInfos = new Array();
        configusrations.forEach((cors) => {
            cors = this.resolveReferences(cors, pipelineConfig);
            const corsInfo = {};
            const corsOptions = this.configureCorsOptions(cors);
            corsInfo.corsMiddleware = corsLib(corsOptions);
            if (this.logger.isDebugEnabled()) {
                this.logger.debug(`Configuring Cors for path [${api.path}].`);
            }
            if (cors.group) {
                if (this.logger.isDebugEnabled()) {
                    const groups = Groups.filter(api.group, cors.group);
                    this.logger.debug(`Configuring Group filters for Cors on path [${api.path}]. Groups [${JSON.stringify(groups)}]`);
                }
                corsInfo.groupValidator = Groups.buildGroupAllowFilter(api.group, cors.group);
            }
            corsInfos.push(corsInfo);
        });
        this.setupMiddlewares(apiRouter, corsInfos);
    }
    configureCorsOptions(cors) {
        const corsOptions = _.omit(cors, 'id', 'origin', 'maxAge', 'group');
        if (cors.maxAge) {
            corsOptions.maxAge = time_intervals_1.getMilisecondsInterval(cors.maxAge);
        }
        if (cors.origin.enableAll) {
            corsOptions.origin = true;
        }
        else if (cors.origin.disableAll) {
            corsOptions.origin = false;
        }
        else if (cors.origin.allow) {
            if (_.filter(cors.origin.allow, obj => obj.value === '*').length > 0) {
                corsOptions.origin = true;
            }
            else {
                corsOptions.origin = cors.origin.allow.map(originConfig => {
                    if (originConfig.value) {
                        return originConfig.value;
                    }
                    else {
                        return new RegExp(originConfig.regexp);
                    }
                });
            }
        }
        else if (cors.origin.middleware) {
            const corsMiddleware = this.middlewareLoader.loadMiddleware('cors/origin', cors.origin.middleware);
            corsOptions.origin = (origin, callback) => {
                Promise.resolve(corsMiddleware(origin))
                    .then(result => {
                    callback(null, result);
                }).catch(error => {
                    callback(error);
                });
            };
        }
        return corsOptions;
    }
    resolveReferences(cors, pipelineConfig) {
        if (cors.use && pipelineConfig.cors) {
            if (pipelineConfig.cors[cors.use]) {
                cors = _.defaults(cors, pipelineConfig.cors[cors.use]);
            }
            else {
                throw new Error(`Invalid reference ${cors.use}. There is no configuration for this id.`);
            }
        }
        return cors;
    }
    setupMiddlewares(apiRouter, corsInfos) {
        corsInfos.forEach((corsInfo) => {
            const middleware = this.buildMiddleware(corsInfo);
            apiRouter.use(middleware);
        });
    }
    buildMiddleware(corsInfo) {
        return (req, res, next) => {
            if (corsInfo.groupValidator) {
                if (corsInfo.groupValidator(req, res)) {
                    corsInfo.corsMiddleware(req, res, next);
                }
                else {
                    next();
                }
            }
            else {
                corsInfo.corsMiddleware(req, res, next);
            }
        };
    }
    sortMiddlewares(corsConfigs, path) {
        const generalThrottlings = _.filter(corsConfigs, (value) => {
            if (value.group) {
                return true;
            }
            return false;
        });
        if (generalThrottlings.length > 1) {
            this.logger.error(`Invalid throttling configuration for api [${path}]. Conflicting configurations for default group`);
            return [];
        }
        if (generalThrottlings.length > 0) {
            const index = corsConfigs.indexOf(generalThrottlings[0]);
            if (index < corsConfigs.length - 1) {
                const gen = corsConfigs.splice(index, 1);
                corsConfigs.push(gen[0]);
            }
        }
        return corsConfigs;
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ApiCors.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_loader_1.MiddlewareLoader)
], ApiCors.prototype, "middlewareLoader", void 0);
ApiCors = __decorate([
    typescript_ioc_1.AutoWired
], ApiCors);
exports.ApiCors = ApiCors;
//# sourceMappingURL=cors.js.map