'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const jsonata_1 = require("../../../config/jsonata");
const logger_1 = require("../../../logger");
const typescript_ioc_1 = require("typescript-ioc");
const jsonata = require('jsonata');
module.exports = function (config) {
    jsonata_1.validateJsonAtaExpression(config);
    const logger = typescript_ioc_1.Container.get(logger_1.Logger);
    const expression = jsonata(config.expression);
    return (err, req, res, next) => {
        if (err && err.message) {
            if (res.headersSent) {
                return next(err);
            }
            res.set('Content-Type', 'application/json');
            res.status(err.statusCode || err.status || 500);
            let body;
            try {
                body = expression.evaluate({
                    error: err,
                    req: req,
                    res: res
                });
            }
            catch (e) {
                body = { error: err.message };
            }
            res.json(body);
            if (logger.isWarnEnabled()) {
                logger.warn(`Error on API pipeline processing: ${err.message}`);
            }
        }
        else {
            next(err);
        }
    };
};
module.exports.factory = true;
//# sourceMappingURL=json.js.map