'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const path_1 = require("../utils/path");
const functions_1 = require("../utils/functions");
const mm = require("micromatch");
function filter(groups, groupIds) {
    const filtered = _.filter(groups, (g) => {
        return groupIds.indexOf(g.id) >= 0;
    });
    return filtered;
}
exports.filter = filter;
function buildGroupAllowFilter(groups, groupIds) {
    const body = `return ${buildGroupAllowTest('req', groups, groupIds)};`;
    return functions_1.createFunction({ mm: mm }, 'req', 'res', body);
}
exports.buildGroupAllowFilter = buildGroupAllowFilter;
function buildGroupAllowTest(request, groups, groupIds) {
    const func = new Array();
    const filtered = filter(groups, groupIds);
    filtered.forEach((group, index) => {
        if (index > 0) {
            func.push(`||`);
        }
        func.push(`(`);
        group.member.forEach((member, memberIndex) => {
            if (memberIndex > 0) {
                func.push(`||`);
            }
            func.push(`(`);
            let hasMethodFilter = false;
            if (member.method && member.method.length > 0) {
                func.push(`(`);
                hasMethodFilter = true;
                member.method.forEach((method, i) => {
                    if (i > 0) {
                        func.push(`||`);
                    }
                    func.push(`(${request}.method === '${method.toUpperCase()}')`);
                });
                func.push(`)`);
            }
            let hasProtocolFilter = false;
            if (member.protocol && member.protocol.length > 0) {
                if (hasMethodFilter) {
                    func.push(`&&`);
                }
                func.push(`(`);
                hasProtocolFilter = true;
                member.protocol.forEach((protocol, i) => {
                    if (i > 0) {
                        func.push(`||`);
                    }
                    func.push(`(${request}.protocol === '${protocol.toLowerCase()}')`);
                });
                func.push(`)`);
            }
            if (member.path && member.path.length > 0) {
                if (hasMethodFilter || hasProtocolFilter) {
                    func.push(`&&`);
                }
                func.push(`(`);
                member.path.forEach((path, i) => {
                    if (i > 0) {
                        func.push(`||`);
                    }
                    func.push(`(mm.isMatch(${request}.path, '${path_1.normalizePath(path)}'))`);
                });
                func.push(`)`);
            }
            func.push(`)`);
        });
        func.push(`)`);
    });
    return func.join('');
}
exports.buildGroupAllowTest = buildGroupAllowTest;
function buildGroupDenyFilter(groups, names) {
    const body = `return ${buildGroupDenyTest('req', groups, names)}`;
    return functions_1.createFunction({ mm: mm }, 'req', 'res', body);
}
exports.buildGroupDenyFilter = buildGroupDenyFilter;
function buildGroupDenyTest(request, groups, names) {
    const func = new Array();
    const filtered = filter(groups, names);
    filtered.forEach((group, index) => {
        if (index > 0) {
            func.push(`&&`);
        }
        func.push(`(`);
        group.member.forEach((member, memberIndex) => {
            if (memberIndex > 0) {
                func.push(`&&`);
            }
            func.push(`(`);
            let hasMethodFilter = false;
            if (member.method && member.method.length > 0) {
                func.push(`(`);
                hasMethodFilter = true;
                member.method.forEach((method, i) => {
                    if (i > 0) {
                        func.push(`&&`);
                    }
                    func.push(`(${request}.method !== '${method.toUpperCase()}')`);
                });
                func.push(`)`);
            }
            let hasProtocolFilter = false;
            if (member.protocol && member.protocol.length > 0) {
                if (hasMethodFilter) {
                    func.push(`&&`);
                }
                func.push(`(`);
                hasProtocolFilter = true;
                member.protocol.forEach((protocol, i) => {
                    if (i > 0) {
                        func.push(`&&`);
                    }
                    func.push(`(${request}.protocol !== '${protocol.toLowerCase()}')`);
                });
                func.push(`)`);
            }
            if (member.path && member.path.length > 0) {
                if (hasMethodFilter || hasProtocolFilter) {
                    func.push(`&&`);
                }
                func.push(`(`);
                member.path.forEach((path, i) => {
                    if (i > 0) {
                        func.push(`&&`);
                    }
                    func.push(`!(mm.isMatch(${request}.path, '${path_1.normalizePath(path)}'))`);
                });
                func.push(`)`);
            }
            func.push(`)`);
        });
        func.push(`)`);
    });
    return func.join('');
}
exports.buildGroupDenyTest = buildGroupDenyTest;
function buildGroupNotAllowFilter(groups, groupIds) {
    const body = `return ${buildGroupNotAllowTest('req', groups, groupIds)};`;
    return functions_1.createFunction({ mm: mm }, 'req', 'res', body);
}
exports.buildGroupNotAllowFilter = buildGroupNotAllowFilter;
function buildGroupNotAllowTest(request, groups, names) {
    const func = new Array();
    const filtered = filter(groups, names);
    filtered.forEach((group, index) => {
        if (index > 0) {
            func.push(`||`);
        }
        func.push(`(`);
        group.member.forEach((member, memberIndex) => {
            if (memberIndex > 0) {
                func.push(`&&`);
            }
            func.push(`(`);
            let hasMethodFilter = false;
            if (member.method && member.method.length > 0) {
                func.push(`(`);
                hasMethodFilter = true;
                member.method.forEach((method, i) => {
                    if (i > 0) {
                        func.push(`&&`);
                    }
                    func.push(`(${request}.method !== '${method.toUpperCase()}')`);
                });
                func.push(`)`);
            }
            let hasProtocolFilter = false;
            if (member.protocol && member.protocol.length > 0) {
                if (hasMethodFilter) {
                    func.push(`||`);
                }
                func.push(`(`);
                hasProtocolFilter = true;
                member.protocol.forEach((protocol, i) => {
                    if (i > 0) {
                        func.push(`&&`);
                    }
                    func.push(`(${request}.protocol !== '${protocol.toLowerCase()}')`);
                });
                func.push(`)`);
            }
            if (member.path && member.path.length > 0) {
                if (hasMethodFilter || hasProtocolFilter) {
                    func.push(`||`);
                }
                func.push(`(`);
                member.path.forEach((path, mIndex) => {
                    if (mIndex > 0) {
                        func.push(`&&`);
                    }
                    func.push(`!(mm.isMatch(${request}.path, '${path_1.normalizePath(path)}'))`);
                });
                func.push(`)`);
            }
            func.push(`)`);
        });
        func.push(`)`);
    });
    return func.join('');
}
exports.buildGroupNotAllowTest = buildGroupNotAllowTest;
//# sourceMappingURL=group.js.map