'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const _ = require("lodash");
const errors_1 = require("../../../config/errors");
const responseHeadersSchema = Joi.object().keys({
    headers: Joi.object().unknown(true),
    removeHeaders: Joi.alternatives([Joi.array().items(Joi.string()), Joi.string()]),
    updateHeaders: Joi.object().unknown(true)
});
function validateResponseHeadersConfig(config) {
    const result = Joi.validate(config, responseHeadersSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
module.exports = function (config) {
    validateResponseHeadersConfig(config);
    config.removeHeaders = _.castArray(config.removeHeaders);
    const updateHeaders = config.updateHeaders || config.headers;
    return (body, headers, request) => {
        const result = { body: body };
        if (config.removeHeaders && config.removeHeaders.length) {
            result.removeHeaders = config.removeHeaders;
        }
        if (updateHeaders) {
            result.updateHeaders = updateHeaders;
        }
        return result;
    };
};
module.exports.factory = true;
//# sourceMappingURL=responseHeaders.js.map