'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const errors_1 = require("../../../config/errors");
const _ = require("lodash");
const destinationConfig = Joi.object().keys({
    target: Joi.string().required(),
    value: Joi.string().required()
});
const headerSplitConfigSchema = Joi.object().keys({
    defaultTarget: Joi.string().required(),
    destinations: Joi.alternatives([Joi.array().items(destinationConfig).min(1), destinationConfig]).required(),
    name: Joi.string().required()
});
function validateHeaderSplitConfig(config) {
    const result = Joi.validate(config, headerSplitConfigSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
module.exports = function (config) {
    validateHeaderSplitConfig(config);
    const targets = new Map();
    config.destinations = _.castArray(config.destinations || []);
    config.destinations.forEach(destination => {
        targets.set(destination.value, destination.target);
    });
    return (req) => {
        const param = req.headers[config.name];
        let target = targets.get(param);
        if (!target) {
            target = config.defaultTarget;
        }
        return target;
    };
};
module.exports.factory = true;
//# sourceMappingURL=header.js.map