'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const chooser = require("weighted");
const errors_1 = require("../../../config/errors");
const _ = require("lodash");
const destinationConfig = Joi.object().keys({
    target: Joi.string().required(),
    weight: Joi.number().required()
});
const trafficSplitConfigSchema = Joi.object().keys({
    destinations: Joi.alternatives([Joi.array().items(destinationConfig).min(1), destinationConfig]).required()
});
function validateTrafficSplitConfig(config) {
    const result = Joi.validate(config, trafficSplitConfigSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
module.exports = function (config) {
    validateTrafficSplitConfig(config);
    config.destinations = _.castArray(config.destinations || []);
    const values = config.destinations.map(destination => destination.target);
    const weights = config.destinations.map(destination => destination.weight);
    return (req) => {
        return chooser.select(values, weights);
    };
};
module.exports.factory = true;
//# sourceMappingURL=trafficSplit.js.map