'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const consul = require("consul");
const errors_1 = require("../../../../config/errors");
const time_intervals_1 = require("../../../../utils/time-intervals");
const fs = require("fs-extra-promise");
const _ = require("lodash");
const defaultsConsulConfigSchema = Joi.object().keys({
    consistent: Joi.boolean(),
    dc: Joi.string(),
    index: Joi.string(),
    stale: Joi.boolean(),
    token: Joi.string(),
    wait: Joi.alternatives([Joi.string(), Joi.number().positive()]),
    wan: Joi.boolean()
});
const consulConfigSchema = Joi.object().keys({
    ca: Joi.alternatives([Joi.array().items(Joi.string()), Joi.string()]),
    defaults: defaultsConsulConfigSchema,
    host: Joi.string(),
    port: Joi.alternatives([Joi.string(), Joi.number().positive()]),
    secure: Joi.boolean()
});
function validateConsulConfig(config) {
    const result = Joi.validate(config, consulConfigSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
module.exports = function (config) {
    validateConsulConfig(config);
    const consulConfig = {};
    if (config.host) {
        consulConfig.host = config.host;
    }
    if (config.port) {
        consulConfig.port = `${config.port}`;
    }
    if (config.secure) {
        consulConfig.secure = config.secure;
    }
    if (config.defaults) {
        if (config.defaults.wait) {
            config.defaults.wait = `${time_intervals_1.getMilisecondsInterval(config.defaults.wait)}ms`;
        }
        consulConfig.defaults = config.defaults;
    }
    if (config.ca) {
        config.ca = _.castArray(config.ca);
        consulConfig.ca = config.ca.map(ca => fs.readFileSync(ca));
    }
    return () => {
        return consul(consulConfig);
    };
};
module.exports.factory = true;
//# sourceMappingURL=consul.js.map