'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../../logger");
const typescript_ioc_1 = require("typescript-ioc");
const middleware_loader_1 = require("../../utils/middleware-loader");
let ServiceDiscovery = class ServiceDiscovery {
    constructor() {
        this.loadedClients = new Map();
    }
    async loadServiceDiscoveryProviders(gatewayConfig) {
        if (!gatewayConfig.serviceDiscovery || !gatewayConfig.serviceDiscovery.provider || !gatewayConfig.serviceDiscovery.provider.length) {
            return;
        }
        const promises = gatewayConfig.serviceDiscovery.provider.map(provider => {
            return new Promise((resolve, reject) => {
                const middlewareId = this.middlewareLoader.getId(provider);
                try {
                    const middleware = this.middlewareLoader.loadMiddleware('servicediscovery/provider', provider);
                    Promise.resolve(middleware())
                        .then((serviceDiscovery) => {
                        this.loadedClients.set(middlewareId, serviceDiscovery);
                        this.logger.info(`Service Discovery provider ${middlewareId} initialized.`);
                        resolve();
                    })
                        .catch(error => {
                        reject(new Error(`Error loading service discovery provider ${middlewareId}. Error: ${error.message}`));
                    });
                }
                catch (error) {
                    reject(new Error(`Error loading service discovery provider ${middlewareId}. Error: ${error.message}`));
                }
            });
        });
        await Promise.all(promises);
    }
    loadServiceDiscovery(middlewareConfig, ssl) {
        const provider = this.middlewareLoader.getId(middlewareConfig);
        const serviceDiscovery = this.loadedClients.get(provider);
        if (serviceDiscovery) {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug(`Loading service discovery middleware ${provider}.`);
            }
            middlewareConfig.options = middlewareConfig.options || {};
            middlewareConfig.options.clientAgent = serviceDiscovery;
            if (ssl) {
                middlewareConfig.options.ssl = ssl;
            }
            return this.middlewareLoader.loadMiddleware('servicediscovery', middlewareConfig);
        }
        else {
            throw new Error(`Error loading service discovery. Provider ${provider} does not exists. Ensure you have configured a provider.`);
        }
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ServiceDiscovery.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_loader_1.MiddlewareLoader)
], ServiceDiscovery.prototype, "middlewareLoader", void 0);
ServiceDiscovery = __decorate([
    typescript_ioc_1.Singleton
], ServiceDiscovery);
exports.ServiceDiscovery = ServiceDiscovery;
//# sourceMappingURL=service-discovery.js.map