'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const typescript_ioc_1 = require("typescript-ioc");
const database_1 = require("../../../database");
const errors_1 = require("../../../config/errors");
const LOG_KEY = '{REQUEST_LOG}';
const requestLogSchema = Joi.object().keys({
    maxEntries: Joi.number()
});
function validateRequestLogConfig(config) {
    const result = Joi.validate(config, requestLogSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
module.exports = (config) => {
    validateRequestLogConfig(config);
    const database = typescript_ioc_1.Container.get(database_1.Database);
    return (requestLog) => {
        database.redisClient.multi()
            .lpush(LOG_KEY, JSON.stringify(requestLog))
            .ltrim(LOG_KEY, 0, config.maxEntries || 10000)
            .exec();
    };
};
module.exports.factory = true;
//# sourceMappingURL=redis.js.map