'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const Groups = require("../group");
const redis_store_1 = require("./redis-store");
const logger_1 = require("../../logger");
const typescript_ioc_1 = require("typescript-ioc");
const time_intervals_1 = require("../../utils/time-intervals");
const middleware_loader_1 = require("../../utils/middleware-loader");
const errors_1 = require("../error/errors");
const request_1 = require("../stats/request");
let ApiRateLimit = class ApiRateLimit {
    throttling(apiRouter, api, pipelineConfig) {
        const path = api.path;
        const throttlings = this.sortLimiters(api.throttling, path);
        const rateLimit = require('express-rate-limit');
        const throttlingInfos = new Array();
        throttlings.forEach((throttling) => {
            throttling = this.resolveReferences(throttling, pipelineConfig);
            const throttlingInfo = {};
            const rateConfig = _.defaults(_.omit(throttling, 'store', 'keyGenerator', 'handler', 'group', 'timeWindow', 'delay'), {
                message: 'Too many requests, please try again later.',
                statusCode: 429
            });
            rateConfig.windowMs = time_intervals_1.getMilisecondsInterval(throttling.timeWindow, 60000);
            rateConfig.delayMs = time_intervals_1.getMilisecondsInterval(throttling.delay, 0);
            rateConfig.store = new redis_store_1.RedisStore({
                expire: (rateConfig.windowMs / 1000) + 1,
                path: path
            });
            if (throttling.keyGenerator) {
                rateConfig.keyGenerator = this.middlewareLoader.loadMiddleware('throttling/keyGenerator', throttling.keyGenerator);
            }
            if (throttling.skip) {
                rateConfig.skip = this.middlewareLoader.loadMiddleware('throttling/skip', throttling.skip);
            }
            this.configureThrottlingHandlerFunction(path, throttling, rateConfig, api);
            throttlingInfo.limiter = new rateLimit(rateConfig);
            if (this.logger.isDebugEnabled()) {
                this.logger.debug(`Configuring Throtlling controller for path [${api.path}].`);
            }
            if (throttling.group) {
                if (this.logger.isDebugEnabled()) {
                    const groups = Groups.filter(api.group, throttling.group);
                    this.logger.debug(`Configuring Group filters for Throtlling on path [${api.path}]. Groups [${JSON.stringify(groups)}]`);
                }
                throttlingInfo.groupValidator = Groups.buildGroupAllowFilter(api.group, throttling.group);
            }
            throttlingInfos.push(throttlingInfo);
        });
        this.setupMiddlewares(apiRouter, throttlingInfos);
    }
    resolveReferences(throttling, pipelineConfig) {
        if (throttling.use && pipelineConfig.throttling) {
            if (pipelineConfig.throttling[throttling.use]) {
                throttling = _.defaults(throttling, pipelineConfig.throttling[throttling.use]);
            }
            else {
                throw new Error(`Invalid reference ${throttling.use}. There is no configuration for this id.`);
            }
        }
        return throttling;
    }
    configureThrottlingHandlerFunction(path, throttling, rateConfig, api) {
        if (this.requestLogger.isRequestLogEnabled(api)) {
            if (throttling.handler) {
                const customHandler = this.middlewareLoader.loadMiddleware('throttling/handler', throttling.handler);
                rateConfig.handler = (req, res, next) => {
                    const requestLog = this.requestLogger.getRequestLog(req);
                    if (requestLog) {
                        requestLog.throttling = 'exceeded';
                    }
                    customHandler(req, res, next);
                };
            }
            else {
                rateConfig.handler = (req, res, next) => {
                    const requestLog = this.requestLogger.getRequestLog(req);
                    if (requestLog) {
                        requestLog.throttling = 'exceeded';
                    }
                    next(new errors_1.ProxyError(rateConfig.message, rateConfig.statusCode));
                };
            }
        }
        else if (throttling.handler) {
            rateConfig.handler = this.middlewareLoader.loadMiddleware('throttling/handler', throttling.handler);
        }
    }
    setupMiddlewares(apiRouter, throttlingInfos) {
        throttlingInfos.forEach((throttlingInfo) => {
            apiRouter.use(this.buildMiddleware(throttlingInfo));
        });
    }
    buildMiddleware(throttlingInfo) {
        return (req, res, next) => {
            if (throttlingInfo.groupValidator) {
                if (throttlingInfo.groupValidator(req, res)) {
                    throttlingInfo.limiter(req, res, next);
                }
                else {
                    next();
                }
            }
            else {
                throttlingInfo.limiter(req, res, next);
            }
        };
    }
    sortLimiters(throttlings, path) {
        const generalThrottlings = _.filter(throttlings, (value) => {
            if (value.group) {
                return true;
            }
            return false;
        });
        if (generalThrottlings.length > 1) {
            this.logger.error(`Invalid throttling configuration for api [${path}]. Conflicting configurations for default group`);
            return [];
        }
        if (generalThrottlings.length > 0) {
            const index = throttlings.indexOf(generalThrottlings[0]);
            if (index < throttlings.length - 1) {
                const gen = throttlings.splice(index, 1);
                throttlings.push(gen[0]);
            }
        }
        return throttlings;
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ApiRateLimit.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_loader_1.MiddlewareLoader)
], ApiRateLimit.prototype, "middlewareLoader", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", request_1.RequestLogger)
], ApiRateLimit.prototype, "requestLogger", void 0);
ApiRateLimit = __decorate([
    typescript_ioc_1.AutoWired
], ApiRateLimit);
exports.ApiRateLimit = ApiRateLimit;
//# sourceMappingURL=throttling.js.map