'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const api_1 = require("../api");
const events_1 = require("../../config/events");
const logger_1 = require("../../logger");
const typescript_ioc_1 = require("typescript-ioc");
const database_1 = require("../../database");
const middleware_installer_1 = require("../../utils/middleware-installer");
const events_2 = require("events");
const machine_1 = require("../../utils/machine");
class Constants {
}
Constants.MIDDLEWARE_INSTALLATION = '{middleware_installation}';
let RedisConfigService = class RedisConfigService extends events_2.EventEmitter {
    constructor() {
        super(...arguments);
        this.subscribed = false;
    }
    getApiConfig(apiId) {
        return this.apiService.get(apiId);
    }
    getAllApiConfig() {
        return this.apiService.list();
    }
    async subscribeEvents() {
        if (!this.subscribed) {
            const topicPattern = `${events_1.ConfigTopics.BASE_TOPIC}:*`;
            await this.database.redisEvents.psubscribe(topicPattern);
            await this.database.redisEvents.on('pmessage', (pattern, channel, message) => {
                if (this.logger.isDebugEnabled()) {
                    this.logger.debug(`Message Received on topic ${channel}. Message: ${JSON.stringify(message)}`);
                }
                try {
                    const parsedMesg = JSON.parse(message);
                    switch (channel) {
                        case events_1.ConfigTopics.CONFIG_UPDATED:
                            this.emit(events_1.ConfigEvents.CONFIG_UPDATED, parsedMesg.packageId, parsedMesg.needsReload);
                            break;
                        case events_1.ConfigTopics.CIRCUIT_CHANGED:
                            this.emit(events_1.ConfigEvents.CIRCUIT_CHANGED, parsedMesg.id, parsedMesg.state);
                            break;
                        default:
                    }
                }
                catch (err) {
                    this.logger.error(`Error processing received message. Message: ${message}. Err: ${this.logger.inspectObject(err)}`);
                }
            });
            if (this.logger.isDebugEnabled()) {
                this.logger.debug(`Listening to events on topic ${topicPattern}`);
            }
            this.subscribed = true;
        }
    }
    async installAllMiddlewares() {
        const machineId = machine_1.getMachineId();
        const host = os.hostname();
        const idMsg = 'allMiddlewares';
        const replies = await this.database.redisClient.multi()
            .setnx(`${Constants.MIDDLEWARE_INSTALLATION}:${host}`, machineId)
            .setnx(`${Constants.MIDDLEWARE_INSTALLATION}:${host}:${idMsg}`, machineId)
            .expire(`${Constants.MIDDLEWARE_INSTALLATION}:${host}`, 60)
            .expire(`${Constants.MIDDLEWARE_INSTALLATION}:${host}:${idMsg}`, 60)
            .exec();
        if (replies[0][1] === 1 && replies[1][1] === 1) {
            this.database.redisClient.expire(`${Constants.MIDDLEWARE_INSTALLATION}:${host}`, 15);
            this.database.redisClient.expire(`${Constants.MIDDLEWARE_INSTALLATION}:${host}:${idMsg}`, 15);
            await this.middlewareInstaller.installAll();
            await this.database.redisClient.del(`${Constants.MIDDLEWARE_INSTALLATION}:${host}`, `${Constants.MIDDLEWARE_INSTALLATION}:${host}:${idMsg}`);
        }
        else {
            await this.runAfterMiddlewareInstallations(idMsg);
        }
    }
    async runAfterMiddlewareInstallations(idMsg) {
        return new Promise((resolve, reject) => {
            const host = os.hostname();
            let interval;
            interval = setInterval(() => {
                this.database.redisClient.exists(`${Constants.MIDDLEWARE_INSTALLATION}:${host}`, `${Constants.MIDDLEWARE_INSTALLATION}:${host}:${idMsg}`)
                    .then((exists) => {
                    if (exists < 2) {
                        clearInterval(interval);
                        return resolve();
                    }
                }).catch((err) => {
                    clearInterval(interval);
                    return reject(err);
                });
            }, 100);
        });
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], RedisConfigService.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", api_1.ApiService)
], RedisConfigService.prototype, "apiService", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_installer_1.MiddlewareInstaller)
], RedisConfigService.prototype, "middlewareInstaller", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", database_1.Database)
], RedisConfigService.prototype, "database", void 0);
RedisConfigService = __decorate([
    typescript_ioc_1.AutoWired,
    typescript_ioc_1.Singleton
], RedisConfigService);
exports.RedisConfigService = RedisConfigService;
//# sourceMappingURL=config.js.map