'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("../../config/events");
const typescript_ioc_1 = require("typescript-ioc");
const database_1 = require("../../database");
const errors_1 = require("../../pipeline/error/errors");
const config_1 = require("../../utils/config");
const _ = require("lodash");
class RedisGatewayService {
    async remove() {
        await this.database.redisClient.multi()
            .del(RedisGatewayService.GATEWAY_CONFIG_KEY)
            .publish(events_1.ConfigTopics.CONFIG_UPDATED, JSON.stringify({ packageId: RedisGatewayService.ADMIN_API, needsReload: true }))
            .exec();
    }
    async save(config) {
        this.castArrays(config);
        await this.database.redisClient.multi()
            .set(RedisGatewayService.GATEWAY_CONFIG_KEY, JSON.stringify(config))
            .publish(events_1.ConfigTopics.CONFIG_UPDATED, JSON.stringify({ packageId: RedisGatewayService.ADMIN_API, needsReload: true }))
            .exec();
    }
    async read() {
        const config = await this.get();
        if (!config) {
            throw new errors_1.NotFoundError('Config not found.');
        }
        return (config);
    }
    async get() {
        const config = await this.database.redisClient.get(RedisGatewayService.GATEWAY_CONFIG_KEY);
        if (!config) {
            return null;
        }
        return JSON.parse(config);
    }
    registerGatewayVersion() {
        return this.database.registerGatewayVersion();
    }
    /**
     * This function cast all array properties inside gateway configuration to array.
     * It is used to allow user to configure array properties as a single item too.
     * @param gateway Gateway configuration
     */
    castArrays(gateway) {
        config_1.castArray(gateway, 'filter');
        config_1.castArray(gateway, 'admin.filter');
        config_1.castArray(gateway, 'serviceDiscovery.provider');
        config_1.castArray(gateway, 'logger.console.stderrLevels');
        config_1.castArray(gateway, 'accessLogger.console.stderrLevels');
        if (_.has(gateway, 'config.cache')) {
            _.keys(gateway.config.cache).forEach(cacheKey => {
                config_1.castArray(gateway.config.cache[cacheKey], 'server.preserveHeaders');
            });
        }
        if (_.has(gateway, 'config.cors')) {
            _.keys(gateway.config.cors).forEach(corsKey => {
                config_1.castArray(gateway.config.cors[corsKey], 'allowedHeaders');
                config_1.castArray(gateway.config.cors[corsKey], 'exposedHeaders');
                config_1.castArray(gateway.config.cors[corsKey], 'method');
            });
        }
    }
}
RedisGatewayService.GATEWAY_CONFIG_KEY = '{config}:gateway';
RedisGatewayService.ADMIN_API = 'ADMIN_API';
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", database_1.Database)
], RedisGatewayService.prototype, "database", void 0);
exports.RedisGatewayService = RedisGatewayService;
//# sourceMappingURL=gateway.js.map