'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("../../config/events");
const typescript_ioc_1 = require("typescript-ioc");
const database_1 = require("../../database");
const errors_1 = require("../../config/errors");
const errors_2 = require("../../pipeline/error/errors");
class RedisMiddlewareService {
    async list(middleware, filter) {
        let result = await this.database.redisClient.smembers(`${RedisMiddlewareService.MIDDLEWARE_PREFIX}:${middleware}`);
        result = result.filter((middlwareName) => {
            if (filter && !middlwareName.includes(filter)) {
                return false;
            }
            return true;
        });
        return result;
    }
    async add(middleware, name, content) {
        await this.ensureMiddlewareCreateConstraints(middleware, name);
        await this.save(middleware, name, content);
        return name;
    }
    async remove(middleware, name) {
        await this.database.redisClient.multi()
            .srem(`${RedisMiddlewareService.MIDDLEWARE_PREFIX}:${middleware}`, name)
            .del(`${RedisMiddlewareService.MIDDLEWARE_PREFIX}:${middleware}:${name}`)
            .publish(events_1.ConfigTopics.CONFIG_UPDATED, JSON.stringify({ packageId: RedisMiddlewareService.ADMIN_API }))
            .exec();
    }
    async update(middleware, name, content, upsert) {
        await this.ensureMiddlewareUpdateConstraints(middleware, name, upsert);
        await this.save(middleware, name, content);
    }
    read(middleware, name) {
        return this.database.redisClient.get(`${RedisMiddlewareService.MIDDLEWARE_PREFIX}:${middleware}:${name}`);
    }
    async save(middleware, name, content) {
        await this.database.redisClient.multi()
            .sadd(`${RedisMiddlewareService.MIDDLEWARE_PREFIX}:${middleware}`, name)
            .set(`${RedisMiddlewareService.MIDDLEWARE_PREFIX}:${middleware}:${name}`, content)
            .publish(events_1.ConfigTopics.CONFIG_UPDATED, JSON.stringify({ packageId: RedisMiddlewareService.ADMIN_API }))
            .exec();
    }
    async ensureMiddlewareCreateConstraints(middleware, name) {
        const exists = await this.database.redisClient.exists(`${RedisMiddlewareService.MIDDLEWARE_PREFIX}:${middleware}:${name}`);
        if (exists) {
            throw new errors_1.ValidationError(`Middleware ${name} already exists for type ${middleware}.`);
        }
    }
    async ensureMiddlewareUpdateConstraints(middleware, name, upsert) {
        if (!upsert) {
            const exists = await this.database.redisClient.exists(`${RedisMiddlewareService.MIDDLEWARE_PREFIX}:${middleware}:${name}`);
            if (!exists) {
                throw new errors_2.NotFoundError(`Middleware ${name} not found for type ${middleware}.`);
            }
        }
    }
}
RedisMiddlewareService.MIDDLEWARE_PREFIX = '{config}:middleware';
RedisMiddlewareService.ADMIN_API = 'ADMIN_API';
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", database_1.Database)
], RedisMiddlewareService.prototype, "database", void 0);
exports.RedisMiddlewareService = RedisMiddlewareService;
//# sourceMappingURL=middleware.js.map