'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../../logger");
const typescript_ioc_1 = require("typescript-ioc");
const database_1 = require("../../database");
const events_1 = require("events");
class RedisPluginsDataService extends events_1.EventEmitter {
    listConfigurationItems(configKey) {
        if (this.logger.isDebugEnabled()) {
            this.logger.debug(`Checking configuration items for plugin config key: ${RedisPluginsDataService.PREFIX}:${configKey} `);
        }
        return this.database.redisClient.smembers(`${RedisPluginsDataService.PREFIX}:${configKey}`);
    }
    async addConfigurationItem(configKey, value) {
        await this.database.redisClient.sadd(`${RedisPluginsDataService.PREFIX}:${configKey}`, value);
    }
    watchConfigurationItems(configKey, interval) {
        const result = setInterval(() => {
            this.checkConfigurationItens(configKey);
        }, interval);
        process.nextTick(() => {
            this.checkConfigurationItens(configKey);
        });
        return result;
    }
    stopWatchingConfigurationItems(watcherKey) {
        clearInterval(watcherKey);
    }
    checkConfigurationItens(configKey) {
        this.listConfigurationItems(configKey)
            .then((data) => {
            this.emit('changed', configKey, data);
        }).catch((err) => {
            this.logger.error(`Error retrieving configuration items from redis database.`);
            this.logger.inspectObject(err);
        });
    }
}
RedisPluginsDataService.PREFIX = '{plugin-data}';
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], RedisPluginsDataService.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", database_1.Database)
], RedisPluginsDataService.prototype, "database", void 0);
exports.RedisPluginsDataService = RedisPluginsDataService;
//# sourceMappingURL=plugin-data.js.map