'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const https = require("https");
const url = require("url");
const _ = require("lodash");
const events_1 = require("events");
const time_intervals_1 = require("./time-intervals");
const logger_1 = require("../logger");
const typescript_ioc_1 = require("typescript-ioc");
class HealthCheck extends events_1.EventEmitter {
    constructor(options) {
        super();
        this.checks = {};
        this.checking = false;
        this.options = _.defaults(options, {
            checkInterval: 30000,
            failCount: 2,
            waitTimeout: 2000
        });
        if (options && options.servers && options.servers.length > 0) {
            options.servers.forEach((s) => {
                this.checks[s] = {
                    down: false,
                    failCount: 0,
                    lastStatus: ''
                };
            });
            this.start();
        }
    }
    start() {
        if (!this.interval) {
            this.check();
            this.interval = setInterval(() => {
                this.check();
            }, time_intervals_1.getMilisecondsInterval(this.options.checkInterval, 30000));
        }
    }
    stop() {
        if (this.interval) {
            clearInterval(this.interval);
            this.interval = 0;
        }
    }
    isDown(serverUrl) {
        const hc = this.checks[serverUrl];
        if (hc) {
            return hc.down;
        }
        else {
            return new Error(`healthcheck: Unknow server: ${name}`);
        }
    }
    status() {
        return Object.assign({}, this.checks);
    }
    check() {
        if (!this.checking) {
            this.checking = true;
            this.previousChecks = this.status();
            const promises = new Array();
            _.forIn(this.checks, (config, urlServer) => {
                promises.push(this.checkServer(urlServer, config));
            });
            Promise.all(promises)
                .then((result) => {
                this.checking = false;
                if (!_.isEqual(this.previousChecks, this.checks)) {
                    if (this.logger.isDebugEnabled()) {
                        this.logger.debug(`Health state changed: ${JSON.stringify(this.checks)}`);
                    }
                    this.emit('change', this.status());
                }
            }).catch(err => {
                this.checking = false;
            });
        }
    }
    checkServer(urlServer, config) {
        return new Promise((resolve, reject) => {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug(`Checking server health: ${urlServer}`);
            }
            const urlCheck = url.parse(urlServer);
            let ended = false;
            const library = urlCheck.protocol === 'https' ? https : http;
            const request = library.get({
                agent: false,
                host: urlCheck.hostname,
                path: urlCheck.pathname,
                port: urlCheck.port
            }, (response) => {
                let result = new Buffer('');
                if (response.statusCode === 200) {
                    response.on('data', (chunk) => {
                        result = Buffer.concat([result, chunk]);
                    });
                    response.on('end', () => {
                        if (ended) {
                            return null;
                        }
                        config.lastStatus = 'OK';
                        config.failCount = 0;
                        config.down = false;
                        resolve(config);
                    });
                }
                else {
                    config.lastStatus = 'Non 200 HTTP status code';
                    config.failCount += 1;
                    if (config.failCount >= this.options.failCount) {
                        config.down = true;
                        resolve(config);
                    }
                }
            });
            request.on('socket', (socket) => {
                socket.setTimeout(time_intervals_1.getMilisecondsInterval(this.options.waitTimeout, 2000));
                socket.on('timeout', () => {
                    ended = true;
                    request.abort();
                    config.lastStatus = 'Healthcheck timed out';
                    config.failCount += 1;
                    if (config.failCount >= this.options.failCount) {
                        config.down = true;
                    }
                    resolve(config);
                });
            });
            request.on('error', (error) => {
                ended = true;
                config.lastStatus = error.message;
                config.failCount += 1;
                if (config.failCount >= this.options.failCount) {
                    config.down = true;
                }
                resolve(config);
            });
        });
    }
}
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], HealthCheck.prototype, "logger", void 0);
exports.HealthCheck = HealthCheck;
//# sourceMappingURL=health-check.js.map