'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const path = require("path");
const configuration_1 = require("../configuration");
const typescript_ioc_1 = require("typescript-ioc");
let MiddlewareLoader = MiddlewareLoader_1 = class MiddlewareLoader {
    loadMiddleware(type, middlewareConfig) {
        let p;
        const middlewareId = this.getId(middlewareConfig);
        if ((_.has(MiddlewareLoader_1.providedMiddlewares, type)) &&
            (_.has(MiddlewareLoader_1.providedMiddlewares[type], middlewareId))) {
            p = MiddlewareLoader_1.providedMiddlewares[type][middlewareId];
        }
        else {
            p = path.join(this.config.middlewarePath, type, middlewareId);
        }
        let middleware;
        try {
            middleware = require(p);
        }
        catch (e) {
            middleware = require(middlewareId);
        }
        if (middleware.factory) {
            middleware = middleware(middlewareConfig.options || {});
        }
        return middleware;
    }
    getId(middlewareConfig) {
        return middlewareConfig.id || middlewareConfig.name;
    }
};
MiddlewareLoader.providedMiddlewares = {
    'authentication/strategy': {
        'basic': '../pipeline/authentication/strategies/basic',
        'jwt': '../pipeline/authentication/strategies/jwt',
        'local': '../pipeline/authentication/strategies/local'
    },
    'errorhandler': {
        'json': '../pipeline/error/handlers/json',
        'mustache': '../pipeline/error/handlers/mustache'
    },
    'filter': {
        'ipFilter': '../pipeline/filter/filters/ipFilter'
    },
    'interceptor/request': {
        'requestBodyTransformer': '../pipeline/proxy/interceptors/requestBodyTransformer',
        'requestHeaders': '../pipeline/proxy/interceptors/requestHeaders',
        'requestMustache': '../pipeline/proxy/interceptors/requestMustache',
        'requestXml': '../pipeline/proxy/interceptors/requestXml'
    },
    'interceptor/response': {
        'responseBodyTransformer': '../pipeline/proxy/interceptors/responseBodyTransformer',
        'responseHeaders': '../pipeline/proxy/interceptors/responseHeaders',
        'responseMustache': '../pipeline/proxy/interceptors/responseMustache',
        'responseXml': '../pipeline/proxy/interceptors/responseXml',
        'webSecurity': '../pipeline/proxy/interceptors/webSecurity'
    },
    'proxy/router': {
        'header': '../pipeline/proxy/routers/header',
        'loadBalancer': '../pipeline/proxy/routers/loadBalancer',
        'query': '../pipeline/proxy/routers/query',
        'trafficSplit': '../pipeline/proxy/routers/trafficSplit'
    },
    'request/logger': {
        'redis': '../pipeline/stats/request-logger/redis'
    },
    'servicediscovery': {
        'consul': '../pipeline/servicediscovery/middleware/consul'
    },
    'servicediscovery/provider': {
        'consul': '../pipeline/servicediscovery/middleware/provider/consul'
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", configuration_1.Configuration)
], MiddlewareLoader.prototype, "config", void 0);
MiddlewareLoader = MiddlewareLoader_1 = __decorate([
    typescript_ioc_1.AutoWired,
    typescript_ioc_1.Singleton
], MiddlewareLoader);
exports.MiddlewareLoader = MiddlewareLoader;
var MiddlewareLoader_1;
//# sourceMappingURL=middleware-loader.js.map